
#ifndef VTKPVWEBEXPORTER_EXPORT_H
#define VTKPVWEBEXPORTER_EXPORT_H

#ifdef VTKPVWEBEXPORTER_STATIC_DEFINE
#  define VTKPVWEBEXPORTER_EXPORT
#  define VTKPVWEBEXPORTER_NO_EXPORT
#else
#  ifndef VTKPVWEBEXPORTER_EXPORT
#    ifdef PVWebExporter_EXPORTS
        /* We are building this library */
#      define VTKPVWEBEXPORTER_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVWEBEXPORTER_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVWEBEXPORTER_NO_EXPORT
#    define VTKPVWEBEXPORTER_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVWEBEXPORTER_DEPRECATED
#  define VTKPVWEBEXPORTER_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVWEBEXPORTER_DEPRECATED_EXPORT
#  define VTKPVWEBEXPORTER_DEPRECATED_EXPORT VTKPVWEBEXPORTER_EXPORT VTKPVWEBEXPORTER_DEPRECATED
#endif

#ifndef VTKPVWEBEXPORTER_DEPRECATED_NO_EXPORT
#  define VTKPVWEBEXPORTER_DEPRECATED_NO_EXPORT VTKPVWEBEXPORTER_NO_EXPORT VTKPVWEBEXPORTER_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVWEBEXPORTER_NO_DEPRECATED
#    define VTKPVWEBEXPORTER_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVWEBEXPORTER_EXPORT_H */
