
#ifndef VTKPVCORE_EXPORT_H
#define VTKPVCORE_EXPORT_H

#ifdef VTKPVCORE_STATIC_DEFINE
#  define VTKPVCORE_EXPORT
#  define VTKPVCORE_NO_EXPORT
#else
#  ifndef VTKPVCORE_EXPORT
#    ifdef Core_EXPORTS
        /* We are building this library */
#      define VTKPVCORE_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVCORE_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVCORE_NO_EXPORT
#    define VTKPVCORE_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVCORE_DEPRECATED
#  define VTKPVCORE_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVCORE_DEPRECATED_EXPORT
#  define VTKPVCORE_DEPRECATED_EXPORT VTKPVCORE_EXPORT VTKPVCORE_DEPRECATED
#endif

#ifndef VTKPVCORE_DEPRECATED_NO_EXPORT
#  define VTKPVCORE_DEPRECATED_NO_EXPORT VTKPVCORE_NO_EXPORT VTKPVCORE_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVCORE_NO_DEPRECATED
#    define VTKPVCORE_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVCORE_EXPORT_H */
