
#ifndef VTKPVCLIENTSERVERCORECORE_EXPORT_H
#define VTKPVCLIENTSERVERCORECORE_EXPORT_H

#ifdef VTKPVCLIENTSERVERCORECORE_STATIC_DEFINE
#  define VTKPVCLIENTSERVERCORECORE_EXPORT
#  define VTKPVCLIENTSERVERCORECORE_NO_EXPORT
#else
#  ifndef VTKPVCLIENTSERVERCORECORE_EXPORT
#    ifdef ClientServerCoreCore_EXPORTS
        /* We are building this library */
#      define VTKPVCLIENTSERVERCORECORE_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVCLIENTSERVERCORECORE_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVCLIENTSERVERCORECORE_NO_EXPORT
#    define VTKPVCLIENTSERVERCORECORE_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVCLIENTSERVERCORECORE_DEPRECATED
#  define VTKPVCLIENTSERVERCORECORE_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVCLIENTSERVERCORECORE_DEPRECATED_EXPORT
#  define VTKPVCLIENTSERVERCORECORE_DEPRECATED_EXPORT VTKPVCLIENTSERVERCORECORE_EXPORT VTKPVCLIENTSERVERCORECORE_DEPRECATED
#endif

#ifndef VTKPVCLIENTSERVERCORECORE_DEPRECATED_NO_EXPORT
#  define VTKPVCLIENTSERVERCORECORE_DEPRECATED_NO_EXPORT VTKPVCLIENTSERVERCORECORE_NO_EXPORT VTKPVCLIENTSERVERCORECORE_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVCLIENTSERVERCORECORE_NO_DEPRECATED
#    define VTKPVCLIENTSERVERCORECORE_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVCLIENTSERVERCORECORE_EXPORT_H */
