
#ifndef VTKPVPYTHONALGORITHM_EXPORT_H
#define VTKPVPYTHONALGORITHM_EXPORT_H

#ifdef VTKPVPYTHONALGORITHM_STATIC_DEFINE
#  define VTKPVPYTHONALGORITHM_EXPORT
#  define VTKPVPYTHONALGORITHM_NO_EXPORT
#else
#  ifndef VTKPVPYTHONALGORITHM_EXPORT
#    ifdef PythonAlgorithm_EXPORTS
        /* We are building this library */
#      define VTKPVPYTHONALGORITHM_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVPYTHONALGORITHM_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVPYTHONALGORITHM_NO_EXPORT
#    define VTKPVPYTHONALGORITHM_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVPYTHONALGORITHM_DEPRECATED
#  define VTKPVPYTHONALGORITHM_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVPYTHONALGORITHM_DEPRECATED_EXPORT
#  define VTKPVPYTHONALGORITHM_DEPRECATED_EXPORT VTKPVPYTHONALGORITHM_EXPORT VTKPVPYTHONALGORITHM_DEPRECATED
#endif

#ifndef VTKPVPYTHONALGORITHM_DEPRECATED_NO_EXPORT
#  define VTKPVPYTHONALGORITHM_DEPRECATED_NO_EXPORT VTKPVPYTHONALGORITHM_NO_EXPORT VTKPVPYTHONALGORITHM_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVPYTHONALGORITHM_NO_DEPRECATED
#    define VTKPVPYTHONALGORITHM_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVPYTHONALGORITHM_EXPORT_H */
