
#ifndef VTKPVVTKEXTENSIONSDEFAULT_EXPORT_H
#define VTKPVVTKEXTENSIONSDEFAULT_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSDEFAULT_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSDEFAULT_EXPORT
#  define VTKPVVTKEXTENSIONSDEFAULT_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSDEFAULT_EXPORT
#    ifdef VTKExtensionsDefault_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSDEFAULT_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSDEFAULT_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSDEFAULT_NO_EXPORT
#    define VTKPVVTKEXTENSIONSDEFAULT_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSDEFAULT_DEPRECATED
#  define VTKPVVTKEXTENSIONSDEFAULT_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSDEFAULT_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSDEFAULT_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSDEFAULT_EXPORT VTKPVVTKEXTENSIONSDEFAULT_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSDEFAULT_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSDEFAULT_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSDEFAULT_NO_EXPORT VTKPVVTKEXTENSIONSDEFAULT_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSDEFAULT_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSDEFAULT_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSDEFAULT_EXPORT_H */
