
#ifndef VTKPVCLIENTSERVERCOREPYTHONRENDERING_EXPORT_H
#define VTKPVCLIENTSERVERCOREPYTHONRENDERING_EXPORT_H

#ifdef VTKPVCLIENTSERVERCOREPYTHONRENDERING_STATIC_DEFINE
#  define VTKPVCLIENTSERVERCOREPYTHONRENDERING_EXPORT
#  define VTKPVCLIENTSERVERCOREPYTHONRENDERING_NO_EXPORT
#else
#  ifndef VTKPVCLIENTSERVERCOREPYTHONRENDERING_EXPORT
#    ifdef ClientServerCorePythonRendering_EXPORTS
        /* We are building this library */
#      define VTKPVCLIENTSERVERCOREPYTHONRENDERING_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVCLIENTSERVERCOREPYTHONRENDERING_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVCLIENTSERVERCOREPYTHONRENDERING_NO_EXPORT
#    define VTKPVCLIENTSERVERCOREPYTHONRENDERING_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVCLIENTSERVERCOREPYTHONRENDERING_DEPRECATED
#  define VTKPVCLIENTSERVERCOREPYTHONRENDERING_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVCLIENTSERVERCOREPYTHONRENDERING_DEPRECATED_EXPORT
#  define VTKPVCLIENTSERVERCOREPYTHONRENDERING_DEPRECATED_EXPORT VTKPVCLIENTSERVERCOREPYTHONRENDERING_EXPORT VTKPVCLIENTSERVERCOREPYTHONRENDERING_DEPRECATED
#endif

#ifndef VTKPVCLIENTSERVERCOREPYTHONRENDERING_DEPRECATED_NO_EXPORT
#  define VTKPVCLIENTSERVERCOREPYTHONRENDERING_DEPRECATED_NO_EXPORT VTKPVCLIENTSERVERCOREPYTHONRENDERING_NO_EXPORT VTKPVCLIENTSERVERCOREPYTHONRENDERING_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVCLIENTSERVERCOREPYTHONRENDERING_NO_DEPRECATED
#    define VTKPVCLIENTSERVERCOREPYTHONRENDERING_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVCLIENTSERVERCOREPYTHONRENDERING_EXPORT_H */
