
#ifndef VTKPVSERVERIMPLEMENTATIONCORE_EXPORT_H
#define VTKPVSERVERIMPLEMENTATIONCORE_EXPORT_H

#ifdef VTKPVSERVERIMPLEMENTATIONCORE_STATIC_DEFINE
#  define VTKPVSERVERIMPLEMENTATIONCORE_EXPORT
#  define VTKPVSERVERIMPLEMENTATIONCORE_NO_EXPORT
#else
#  ifndef VTKPVSERVERIMPLEMENTATIONCORE_EXPORT
#    ifdef ServerImplementationCore_EXPORTS
        /* We are building this library */
#      define VTKPVSERVERIMPLEMENTATIONCORE_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVSERVERIMPLEMENTATIONCORE_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVSERVERIMPLEMENTATIONCORE_NO_EXPORT
#    define VTKPVSERVERIMPLEMENTATIONCORE_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVSERVERIMPLEMENTATIONCORE_DEPRECATED
#  define VTKPVSERVERIMPLEMENTATIONCORE_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVSERVERIMPLEMENTATIONCORE_DEPRECATED_EXPORT
#  define VTKPVSERVERIMPLEMENTATIONCORE_DEPRECATED_EXPORT VTKPVSERVERIMPLEMENTATIONCORE_EXPORT VTKPVSERVERIMPLEMENTATIONCORE_DEPRECATED
#endif

#ifndef VTKPVSERVERIMPLEMENTATIONCORE_DEPRECATED_NO_EXPORT
#  define VTKPVSERVERIMPLEMENTATIONCORE_DEPRECATED_NO_EXPORT VTKPVSERVERIMPLEMENTATIONCORE_NO_EXPORT VTKPVSERVERIMPLEMENTATIONCORE_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVSERVERIMPLEMENTATIONCORE_NO_DEPRECATED
#    define VTKPVSERVERIMPLEMENTATIONCORE_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVSERVERIMPLEMENTATIONCORE_EXPORT_H */
