
#ifndef VTKPVWEBCORE_EXPORT_H
#define VTKPVWEBCORE_EXPORT_H

#ifdef VTKPVWEBCORE_STATIC_DEFINE
#  define VTKPVWEBCORE_EXPORT
#  define VTKPVWEBCORE_NO_EXPORT
#else
#  ifndef VTKPVWEBCORE_EXPORT
#    ifdef PVWebCore_EXPORTS
        /* We are building this library */
#      define VTKPVWEBCORE_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVWEBCORE_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVWEBCORE_NO_EXPORT
#    define VTKPVWEBCORE_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVWEBCORE_DEPRECATED
#  define VTKPVWEBCORE_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVWEBCORE_DEPRECATED_EXPORT
#  define VTKPVWEBCORE_DEPRECATED_EXPORT VTKPVWEBCORE_EXPORT VTKPVWEBCORE_DEPRECATED
#endif

#ifndef VTKPVWEBCORE_DEPRECATED_NO_EXPORT
#  define VTKPVWEBCORE_DEPRECATED_NO_EXPORT VTKPVWEBCORE_NO_EXPORT VTKPVWEBCORE_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVWEBCORE_NO_DEPRECATED
#    define VTKPVWEBCORE_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVWEBCORE_EXPORT_H */
