
#ifndef VTKPVSERVERMANAGERCORE_EXPORT_H
#define VTKPVSERVERMANAGERCORE_EXPORT_H

#ifdef VTKPVSERVERMANAGERCORE_STATIC_DEFINE
#  define VTKPVSERVERMANAGERCORE_EXPORT
#  define VTKPVSERVERMANAGERCORE_NO_EXPORT
#else
#  ifndef VTKPVSERVERMANAGERCORE_EXPORT
#    ifdef ServerManagerCore_EXPORTS
        /* We are building this library */
#      define VTKPVSERVERMANAGERCORE_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVSERVERMANAGERCORE_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVSERVERMANAGERCORE_NO_EXPORT
#    define VTKPVSERVERMANAGERCORE_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVSERVERMANAGERCORE_DEPRECATED
#  define VTKPVSERVERMANAGERCORE_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVSERVERMANAGERCORE_DEPRECATED_EXPORT
#  define VTKPVSERVERMANAGERCORE_DEPRECATED_EXPORT VTKPVSERVERMANAGERCORE_EXPORT VTKPVSERVERMANAGERCORE_DEPRECATED
#endif

#ifndef VTKPVSERVERMANAGERCORE_DEPRECATED_NO_EXPORT
#  define VTKPVSERVERMANAGERCORE_DEPRECATED_NO_EXPORT VTKPVSERVERMANAGERCORE_NO_EXPORT VTKPVSERVERMANAGERCORE_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVSERVERMANAGERCORE_NO_DEPRECATED
#    define VTKPVSERVERMANAGERCORE_NO_DEPRECATED
#  endif
#endif

/* AutoInit implementations. */
#ifdef vtkPVServerManagerCore_AUTOINIT_INCLUDE
#include vtkPVServerManagerCore_AUTOINIT_INCLUDE
#endif
#ifdef vtkPVServerManagerCore_AUTOINIT
#include "vtkAutoInit.h"
VTK_MODULE_AUTOINIT(vtkPVServerManagerCore)
#endif

#endif /* VTKPVSERVERMANAGERCORE_EXPORT_H */
