
#ifndef VTKPVSERVERMANAGERRENDERING_EXPORT_H
#define VTKPVSERVERMANAGERRENDERING_EXPORT_H

#ifdef VTKPVSERVERMANAGERRENDERING_STATIC_DEFINE
#  define VTKPVSERVERMANAGERRENDERING_EXPORT
#  define VTKPVSERVERMANAGERRENDERING_NO_EXPORT
#else
#  ifndef VTKPVSERVERMANAGERRENDERING_EXPORT
#    ifdef ServerManagerRendering_EXPORTS
        /* We are building this library */
#      define VTKPVSERVERMANAGERRENDERING_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVSERVERMANAGERRENDERING_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVSERVERMANAGERRENDERING_NO_EXPORT
#    define VTKPVSERVERMANAGERRENDERING_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVSERVERMANAGERRENDERING_DEPRECATED
#  define VTKPVSERVERMANAGERRENDERING_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVSERVERMANAGERRENDERING_DEPRECATED_EXPORT
#  define VTKPVSERVERMANAGERRENDERING_DEPRECATED_EXPORT VTKPVSERVERMANAGERRENDERING_EXPORT VTKPVSERVERMANAGERRENDERING_DEPRECATED
#endif

#ifndef VTKPVSERVERMANAGERRENDERING_DEPRECATED_NO_EXPORT
#  define VTKPVSERVERMANAGERRENDERING_DEPRECATED_NO_EXPORT VTKPVSERVERMANAGERRENDERING_NO_EXPORT VTKPVSERVERMANAGERRENDERING_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVSERVERMANAGERRENDERING_NO_DEPRECATED
#    define VTKPVSERVERMANAGERRENDERING_NO_DEPRECATED
#  endif
#endif
/* AutoInit dependencies. */
#include "vtkPVServerManagerCoreModule.h"


/* AutoInit implementations. */
#ifdef vtkPVServerManagerRendering_AUTOINIT_INCLUDE
#include vtkPVServerManagerRendering_AUTOINIT_INCLUDE
#endif
#ifdef vtkPVServerManagerRendering_AUTOINIT
#include "vtkAutoInit.h"
VTK_MODULE_AUTOINIT(vtkPVServerManagerRendering)
#endif

#endif /* VTKPVSERVERMANAGERRENDERING_EXPORT_H */
